/**
Very simple web client.

Orion Sky Lawlor, olawlor@acm.org, 2005/2/8 (Public Domain)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "osl/socket.h"
#include "osl/socket.cpp" /* include body for easy linking */

/* Filename to save web page to */
const char *destfile=NULL;

/** 
  Send this C (null-terminated) string to this network socket.
    Does not send the terminating nul character.
*/
void sendString(SOCKET s,const char *str) {
	skt_sendN(s,str,strlen(str));
}

/**
  Connect to this web server, send the GET request,
  and return a socket to read the web page from.
*/
SOCKET connect_to_URL(const char *url)
{
	char serverAndPort[1000],server[1000],file[1000];
	int port=80;
	/// FIXME: support other URL formats here!
	strcpy(file,"/");
	int nScan=sscanf(url,"http://%999[^/]%999[^%]",serverAndPort,file);
	if (nScan<1) {
		printf("Cannot parse URL '%s'!\n",url);
		exit(1);
	}
	nScan=sscanf(serverAndPort,"%[^:]:%d",server,&port);
	if (nScan<1) {
		printf("Cannot parse URL port '%s'!\n",url);
		exit(1);
	}
	skt_ip_t ip;
	SOCKET s;
	printf("Connecting to %s:%d\n",server,port);
	s=skt_connect(skt_lookup_ip(server),port,10);
	printf("Connected.  Sending request for %s\n",file);
	sendString(s,"GET ");
	if (0) sleep(10); /* put in a big weird delay */
	sendString(s,file);
	sendString(s," HTTP/1.1\r\nHost: ");
	sendString(s,server);
	sendString(s,"\r\n\r\n");
	/* FIXME: send other HTTP headers here, like Content-Type */
	return s;
}

/*
  Receive a web page from this socket.
*/
void receive_page(SOCKET s) 
{
	int data_len=0;
	printf("Request away.  Awaiting headers\n");
	while (1) {
		std::string h=skt_recv_line(s);
		if (h.size()==0) break; /* end of HTTP headers */
		int i=h.find("Content-Length:");
		if (i>=0) sscanf(h.c_str(),"Content-Length: %d",&data_len);
		printf("Got header: %s\n",h.c_str());
	}
	if (data_len==0) {
		printf("Couldn't parse Content-Length from HTTP headers!\n");
		exit(1);
	}
	printf("Waiting for %d bytes of data\n",data_len);
	char *buf=new char[data_len+1];
	skt_recvN(s,buf,data_len);
	if (destfile==NULL) {
		buf[data_len]=0;
		printf("Client received:\n%s\n",buf);
	} else { /* write data to file */
		FILE *f=fopen(destfile,"wb");
		fwrite(buf,1,data_len,f);
		fclose(f);
		exit(0);
	}
}

int main(int argc,char *argv[])
{
	SOCKET s;
	if (argc<=1) {
		printf("usage: webclient -O <file> <url>\n");
	}
	const char *url=argv[1];
	if (argc>3) {
		destfile=argv[2];
		url=argv[3];
	}
	skt_init();
	s=connect_to_URL(url);
	receive_page(s);
	return 0;
}

